@@ -249,14 +249,21 @@ if_mtu_os(const struct interface *ifp)
 }
 
 int
-if_getssid(struct interface *ifp)
+if_getssid(__unused struct interface *ifp)
 {
 
-	UNUSED(ifp);
 	errno = ENOTSUP;
 	return -1;
 }
 
+/* XXX work out TAP interfaces? */
+bool
+if_ignore(__unused struct dhcpcd_ctx *ctx, __unused const char *ifname)
+{
+
+	return false;
+}
+
 unsigned short
 if_vlanid(__unused const struct interface *ifp)
 {
